/* @flow */
import crypto from 'crypto';

import pProps from 'p-props';
import warning from 'warning';
import { SlackOAuthClient } from 'messaging-api-slack';

import SlackContext from '../context/SlackContext';
import SlackEvent, {
  type EventAPITypes,
  type InteractiveMessageEvent,
  type Message,
  type SlackRawEvent,
} from '../context/SlackEvent';
import { type Session } from '../session/Session';

import { type Connector } from './Connector';

// FIXME
export type SlackUser = {
  id: string,
};

type EventsAPIBody = {
  token: string,
  team_id: string,
  api_app_id: string,
  type: EventAPITypes,
  event: Message,
  authed_users: Array<string>,
  event_id: string,
  event_time: number,
};

export type SlackRequestBody = EventsAPIBody | { payload: string };

type ConstructorOptions = {|
  accessToken?: string,
  client?: SlackOAuthClient,
  verificationToken?: string,
  origin?: string,
  skipProfile?: ?boolean,
|};

export default class SlackConnector implements Connector<SlackRequestBody> {
  _client: SlackOAuthClient;

  _verificationToken: string;

  _skipProfile: boolean;

  constructor({
    accessToken,
    client,
    verificationToken,
    origin,
    skipProfile,
  }: ConstructorOptions) {
    this._client =
      client ||
      SlackOAuthClient.connect({
        accessToken,
        origin,
      });
    this._verificationToken = verificationToken || '';

    // FIXME: maybe set this default value as true
    this._skipProfile = typeof skipProfile === 'boolean' ? skipProfile : false;

    if (!this._verificationToken) {
      warning(
        false,
        '`verificationToken` is not set. Will bypass Slack event verification.\nPass in `verificationToken` to perform Slack event verification.'
      );
    }
  }

  _getRawEventFromRequest(body: SlackRequestBody): SlackRawEvent {
    if (body.event) {
      return (((body: any): EventsAPIBody).event: Message);
    }

    if (body.payload && typeof body.payload === 'string') {
      return (JSON.parse(body.payload): InteractiveMessageEvent);
    }

    // for RTM WebSocket messages
    return ((body: any): Message);
  }

  _isBotEventRequest(body: SlackRequestBody): boolean {
    const rawEvent = this._getRawEventFromRequest(body);
    return !!(
      rawEvent.bot_id ||
      (rawEvent.subtype && rawEvent.subtype === 'bot_message')
    );
  }

  get platform(): string {
    return 'slack';
  }

  get client(): SlackOAuthClient {
    return this._client;
  }

  getUniqueSessionKey(body: SlackRequestBody): string {
    // FIXME: define types for every slack events
    const rawEvent = (this._getRawEventFromRequest(body): any);

    // For interactive_message format
    if (
      rawEvent.channel &&
      typeof rawEvent.channel === 'object' &&
      rawEvent.channel.id
    ) {
      return rawEvent.channel.id;
    }

    // For pin_added format
    if (rawEvent.channel_id) {
      return rawEvent.channel_id;
    }

    // For reaction_added format
    if (
      rawEvent.item &&
      typeof rawEvent.item === 'object' &&
      typeof rawEvent.item.channel === 'string'
    ) {
      return rawEvent.item.channel;
    }

    return ((rawEvent: any): Message).channel;
  }

  async updateSession(session: Session, body: SlackRequestBody): Promise<void> {
    if (this._isBotEventRequest(body)) {
      return;
    }

    const rawEvent = this._getRawEventFromRequest(body);
    let userFromBody;
    if (rawEvent.type === 'interactive_message') {
      userFromBody = rawEvent.user.id;
    } else {
      userFromBody = ((rawEvent: any): Message).user;
    }

    if (
      typeof session.user === 'object' &&
      session.user &&
      session.user.id &&
      session.user.id === userFromBody
    ) {
      return;
    }
    const channelId = this.getUniqueSessionKey(body);
    const senderId = userFromBody;

    if (!senderId) {
      return;
    }

    if (this._skipProfile) {
      session.user = {
        id: senderId,
        _updatedAt: new Date().toISOString(),
      };

      session.channel = {
        id: channelId,
        _updatedAt: new Date().toISOString(),
      };

      Object.freeze(session.user);
      Object.defineProperty(session, 'user', {
        configurable: false,
        enumerable: true,
        writable: false,
        value: session.user,
      });

      Object.freeze(session.channel);
      Object.defineProperty(session, 'channel', {
        configurable: false,
        enumerable: true,
        writable: false,
        value: session.channel,
      });

      return;
    }

    const promises: Object = {
      sender: this._client.getUserInfo(senderId),
    };

    // TODO: check join or leave events?
    if (
      !session.channel ||
      (session.channel.members &&
        Array.isArray(session.channel.members) &&
        session.channel.members.indexOf(senderId) < 0)
    ) {
      promises.channel = this._client.getConversationInfo(channelId);
      promises.channelMembers = this._client.getAllConversationMembers(
        channelId
      );
    }

    // TODO: how to know if user leave team?
    // TODO: team info shared by all channels?
    if (
      !session.team ||
      (session.team.members &&
        Array.isArray(session.team.members) &&
        session.team.members.indexOf(senderId) < 0)
    ) {
      promises.allUsers = this._client.getAllUserList();
    }

    const results = await pProps(promises);

    // FIXME: refine user
    session.user = {
      id: senderId,
      _updatedAt: new Date().toISOString(),
      ...results.sender,
    };
    Object.freeze(session.user);
    Object.defineProperty(session, 'user', {
      configurable: false,
      enumerable: true,
      writable: false,
      value: session.user,
    });

    if (promises.channel) {
      session.channel = {
        ...results.channel,
        members: results.channelMembers,
        _updatedAt: new Date().toISOString(),
      };

      Object.freeze(session.channel);
      Object.defineProperty(session, 'channel', {
        configurable: false,
        enumerable: true,
        writable: false,
        value: session.channel,
      });
    }

    if (promises.allUsers) {
      session.team = {
        members: results.allUsers,
        _updatedAt: new Date().toISOString(),
      };

      Object.freeze(session.team);
      Object.defineProperty(session, 'team', {
        configurable: false,
        enumerable: true,
        writable: false,
        value: session.team,
      });
    }
  }

  mapRequestToEvents(body: SlackRequestBody): Array<SlackEvent> {
    const rawEvent = this._getRawEventFromRequest(body);

    if (this._isBotEventRequest(body)) {
      return []; // FIXME
    }

    return [new SlackEvent(rawEvent)];
  }

  createContext(params: {
    event: SlackEvent,
    session: ?Session,
    initialState: ?Object,
    requestContext: ?Object,
  }): SlackContext {
    return new SlackContext({
      ...params,
      client: this._client,
    });
  }

  verifySignature(tokenFromBody: string): boolean {
    if (!this._verificationToken) {
      // TODO: deprecate this bypassing
      return true;
    }

    const bufferFromBot = Buffer.from(this._verificationToken);
    const bufferFromBody = Buffer.from(tokenFromBody);

    // return early here if buffer lengths are not equal since timingSafeEqual
    // will throw if buffer lengths are not equal
    if (bufferFromBot.length !== bufferFromBody.length) {
      return false;
    }

    return crypto.timingSafeEqual(bufferFromBot, bufferFromBody);
  }
}
